@@ -1,14 +1,20 @@
-use std::f32::consts::PI;
-
 use bevy::prelude::*;
-use bevy_prototype_lyon::entity::ShapeBundle;
-use bevy_prototype_lyon::prelude::*;
-use bevy_prototype_lyon::shapes::RectangleOrigin;
+use bevy_prototype_lyon::{entity::ShapeBundle, prelude::*};
+use lyon_path::{
+    builder::BorderRadii,
+    math::{Point, Rect, Size},
+    path::Builder,
+    traits::PathBuilder,
+};
 
 use heron_core::{CollisionShape, RigidBody, SensorShape};
-use heron_rapier::convert::IntoBevy;
-use heron_rapier::rapier2d::geometry::{ColliderHandle, ColliderSet, Shape};
+use heron_rapier::{
+    convert::{IntoBevy, IntoRapier},
+    rapier2d::geometry::{ColliderSet, Shape},
+    ColliderHandle,
+};
 
+#[allow(clippy::wildcard_imports)]
 use super::*;
 
 pub(crate) fn systems() -> SystemSet {
@@ -19,9 +25,10 @@ pub(crate) fn systems() -> SystemSet {
 }
 
 fn create_debug_sprites(
-    mut commands: Commands<'_>,
+    mut commands: Commands<'_, '_>,
     colliders: Res<'_, ColliderSet>,
     query: Query<
+        '_,
         '_,
         (
             Entity,
@@ -36,7 +43,7 @@ fn create_debug_sprites(
     debug_color: Res<'_, DebugColor>,
 ) {
     for (entity, body, handle, transform, rigid_body_option, sensor_option) in query.iter() {
-        if let Some(collider) = colliders.get(*handle) {
+        if let Some(collider) = colliders.get(handle.into_rapier()) {
             commands
                 .entity(entity)
                 .with_children(|builder| {
@@ -56,11 +63,12 @@ fn create_debug_sprites(
 }
 
 fn replace_debug_sprite(
-    mut commands: Commands<'_>,
+    mut commands: Commands<'_, '_>,
     mut map: ResMut<'_, DebugEntityMap>,
     colliders: Res<'_, ColliderSet>,
     debug_color: Res<'_, DebugColor>,
     query: Query<
+        '_,
         '_,
         (
             Entity,
@@ -74,9 +82,10 @@ fn replace_debug_sprite(
     >,
 ) {
     for (parent_entity, body, handle, transform, rigid_body_option, sensor_option) in query.iter() {
-        if let (Some(debug_entity), Some(collider)) =
-            (map.remove(&parent_entity), colliders.get(*handle))
-        {
+        if let (Some(debug_entity), Some(collider)) = (
+            map.remove(&parent_entity),
+            colliders.get(handle.into_rapier()),
+        ) {
             commands.entity(debug_entity).despawn();
             commands.entity(parent_entity).with_children(|builder| {
                 builder
@@ -93,13 +102,16 @@ fn replace_debug_sprite(
 }
 
 fn delete_debug_sprite(
-    mut commands: Commands<'_>,
+    mut commands: Commands<'_, '_>,
     mut map: ResMut<'_, DebugEntityMap>,
     removed_bodies: RemovedComponents<'_, CollisionShape>,
+    debug_sprites: Query<'_, '_, Entity, With<IsDebug>>,
 ) {
     for parent_entity in removed_bodies.iter() {
         if let Some(debug_entity) = map.remove(&parent_entity) {
-            commands.entity(debug_entity).despawn();
+            if debug_sprites.get(debug_entity).is_ok() {
+                commands.entity(debug_entity).despawn();
+            }
         }
     }
 }
@@ -111,8 +123,10 @@ fn create_shape(
     transform: GlobalTransform,
 ) -> ShapeBundle {
     base_builder(body, shape).build(
-        ShapeColors::new(color),
-        DrawMode::Fill(FillOptions::default()),
+        DrawMode::Fill(FillMode {
+            color,
+            options: FillOptions::default(),
+        }),
         Transform {
             translation: Vec3::Z,
             scale: transform.scale.recip(),
@@ -121,12 +135,13 @@ fn create_shape(
     )
 }
 
+#[allow(clippy::too_many_lines)]
 fn base_builder(body: &CollisionShape, shape: &dyn Shape) -> GeometryBuilder {
     let mut builder = GeometryBuilder::new();
 
     match body {
         CollisionShape::Sphere { radius } => {
-            builder.add(&shapes::Circle {
+            builder = builder.add(&shapes::Circle {
                 radius: *radius,
                 center: Vec2::ZERO,
             });
@@ -135,37 +150,15 @@ fn base_builder(body: &CollisionShape, shape: &dyn Shape) -> GeometryBuilder {
             half_segment,
             radius,
         } => {
-            let half_segment = *half_segment;
-            let radius = *radius;
-            let mut path = PathBuilder::new();
-            path.move_to(Vec2::new(-radius, half_segment));
-            path.arc(
-                Vec2::new(0.0, half_segment),
-                Vec2::new(radius, radius),
-                -PI,
-                0.0,
-            );
-            path.line_to(Vec2::new(radius, -half_segment));
-            path.arc(
-                Vec2::new(0.0, -half_segment),
-                Vec2::new(radius, radius),
-                -PI,
-                0.0,
-            );
-            path.close();
-            builder.add(&path.build());
+            builder = builder.add(&Capsule {
+                half_segment: *half_segment,
+                radius: *radius,
+            });
         }
         CollisionShape::Cuboid {
             half_extends,
             border_radius,
         } => {
-            // bevy_prototype_lyon doesn't have rounded rectangles yet so we implement our own
-            use lyon_path::{
-                builder::BorderRadii,
-                math::{Point, Rect, Size},
-                path::Builder,
-                traits::PathBuilder,
-            };
             struct RoundedRectangle {
                 width: f32,
                 height: f32,
@@ -191,23 +184,15 @@ fn base_builder(body: &CollisionShape, shape: &dyn Shape) -> GeometryBuilder {
                 }
             }
 
-            if let Some(radius) = border_radius {
-                builder.add(&RoundedRectangle {
-                    width: 2.0 * half_extends.x,
-                    height: 2.0 * half_extends.y,
-                    radius: *radius,
-                });
-            } else {
-                builder.add(&shapes::Rectangle {
-                    origin: RectangleOrigin::Center,
-                    width: 2.0 * half_extends.x,
-                    height: 2.0 * half_extends.y,
-                });
-            }
+            builder = builder.add(&RoundedRectangle {
+                width: 2.0 * half_extends.x,
+                height: 2.0 * half_extends.y,
+                radius: border_radius.unwrap_or(0.0),
+            });
         }
         CollisionShape::ConvexHull { .. } => {
             if let Some(polygon) = shape.as_convex_polygon() {
-                builder.add(&shapes::Polygon {
+                builder = builder.add(&shapes::Polygon {
                     points: polygon.points().into_bevy(),
                     closed: true,
                 });
@@ -218,13 +203,13 @@ fn base_builder(body: &CollisionShape, shape: &dyn Shape) -> GeometryBuilder {
             // around the edges of the polygon that are also taken up by the border radius.
             } else if let Some(polygon) = shape.as_round_convex_polygon() {
                 for point in polygon.base_shape.points() {
-                    builder.add(&shapes::Circle {
+                    builder = builder.add(&shapes::Circle {
                         radius: polygon.border_radius,
                         center: point.into_bevy(),
                     });
                 }
 
-                builder.add(&shapes::Polygon {
+                builder = builder.add(&shapes::Polygon {
                     points: polygon.base_shape.points().into_bevy(),
                     closed: true,
                 });
@@ -251,7 +236,7 @@ fn base_builder(body: &CollisionShape, shape: &dyn Shape) -> GeometryBuilder {
                 points.push(Vec2::new(half_size, min_y));
                 points.push(Vec2::new(-half_size, min_y));
 
-                builder.add(&shapes::Polygon {
+                builder = builder.add(&shapes::Polygon {
                     points,
                     closed: true,
                 });
@@ -267,3 +252,27 @@ fn base_builder(body: &CollisionShape, shape: &dyn Shape) -> GeometryBuilder {
 
     builder
 }
+
+struct Capsule {
+    half_segment: f32,
+    radius: f32,
+}
+
+impl Geometry for Capsule {
+    fn add_geometry(&self, path: &mut Builder) {
+        let ctrl_height = self.radius * (4. / 3.);
+        path.begin(Point::new(-self.radius, self.half_segment));
+        path.cubic_bezier_to(
+            Point::new(-self.radius, self.half_segment + ctrl_height),
+            Point::new(self.radius, self.half_segment + ctrl_height),
+            Point::new(self.radius, self.half_segment),
+        );
+        path.line_to(Point::new(self.radius, -self.half_segment));
+        path.cubic_bezier_to(
+            Point::new(self.radius, -self.half_segment - ctrl_height),
+            Point::new(-self.radius, -self.half_segment - ctrl_height),
+            Point::new(-self.radius, -self.half_segment),
+        );
+        path.end(true);
+    }
+}
